@@ -16,6 +16,7 @@ import sys
 import pki.cli
 import pki.server
 import pki.server.instance
+import pki.server.cli.subsystem
 
 # TODO: auto-populate this map from /usr/share/pki/acme/database
 DATABASE_CLASSES = {
@@ -56,8 +57,8 @@ class ACMECLI(pki.cli.CLI):
 
         self.add_module(ACMECreateCLI())
         self.add_module(ACMERemoveCLI())
-        self.add_module(ACMEDeployCLI())
-        self.add_module(ACMEUndeployCLI())
+        self.add_module(pki.server.cli.subsystem.SubsystemDeployCLI(self))
+        self.add_module(pki.server.cli.subsystem.SubsystemUndeployCLI(self))
 
         self.add_module(ACMEMetadataCLI())
         self.add_module(ACMEDatabaseCLI())
@@ -232,6 +233,9 @@ class ACMEDeployCLI(pki.cli.CLI):
         print('Usage: pki-server acme-deploy [OPTIONS] [name]')
         print()
         print('  -i, --instance <instance ID>       Instance ID (default: pki-tomcat).')
+        print('      --wait                         Wait until started.')
+        print('      --max-wait <seconds>           Maximum wait time (default: 60).')
+        print('      --timeout <seconds>            Connection timeout.')
         print('  -v, --verbose                      Run in verbose mode.')
         print('      --debug                        Run in debug mode.')
         print('      --help                         Show help message.')
@@ -242,6 +246,7 @@ class ACMEDeployCLI(pki.cli.CLI):
         try:
             opts, args = getopt.gnu_getopt(argv, 'i:v', [
                 'instance=',
+                'wait', 'max-wait=', 'timeout=',
                 'verbose', 'debug', 'help'])
 
         except getopt.GetoptError as e:
@@ -251,11 +256,23 @@ class ACMEDeployCLI(pki.cli.CLI):
 
         name = 'acme'
         instance_name = 'pki-tomcat'
+        wait = False
+        max_wait = 60
+        timeout = None
 
         for o, a in opts:
             if o in ('-i', '--instance'):
                 instance_name = a
 
+            elif o == '--wait':
+                wait = True
+
+            elif o == '--max-wait':
+                max_wait = int(a)
+
+            elif o == '--timeout':
+                timeout = int(a)
+
             elif o in ('-v', '--verbose'):
                 logging.getLogger().setLevel(logging.INFO)
 
@@ -287,7 +304,13 @@ class ACMEDeployCLI(pki.cli.CLI):
                                 'acme/webapps/acme')
 
         logger.info('Deploying %s webapp', name)
-        instance.deploy_webapp(name, descriptor, doc_base)
+        instance.deploy_webapp(
+            name,
+            descriptor,
+            doc_base,
+            wait=wait,
+            max_wait=max_wait,
+            timeout=timeout)
 
 
 class ACMEUndeployCLI(pki.cli.CLI):
@@ -300,6 +323,9 @@ class ACMEUndeployCLI(pki.cli.CLI):
         print('Usage: pki-server acme-undeploy [OPTIONS] [name]')
         print()
         print('  -i, --instance <instance ID>       Instance ID (default: pki-tomcat).')
+        print('      --wait                         Wait until stopped.')
+        print('      --max-wait <seconds>           Maximum wait time (default: 60).')
+        print('      --timeout <seconds>            Connection timeout.')
         print('  -v, --verbose                      Run in verbose mode.')
         print('      --debug                        Run in debug mode.')
         print('      --help                         Show help message.')
@@ -310,6 +336,7 @@ class ACMEUndeployCLI(pki.cli.CLI):
         try:
             opts, args = getopt.gnu_getopt(argv, 'i:v', [
                 'instance=',
+                'wait', 'max-wait=', 'timeout=',
                 'verbose', 'debug', 'help'])
 
         except getopt.GetoptError as e:
@@ -319,11 +346,23 @@ class ACMEUndeployCLI(pki.cli.CLI):
 
         name = 'acme'
         instance_name = 'pki-tomcat'
+        wait = False
+        max_wait = 60
+        timeout = None
 
         for o, a in opts:
             if o in ('-i', '--instance'):
                 instance_name = a
 
+            elif o == '--wait':
+                wait = True
+
+            elif o == '--max-wait':
+                max_wait = int(a)
+
+            elif o == '--timeout':
+                timeout = int(a)
+
             elif o in ('-v', '--verbose'):
                 logging.getLogger().setLevel(logging.INFO)
 
@@ -350,7 +389,11 @@ class ACMEUndeployCLI(pki.cli.CLI):
         instance.load()
 
         logger.info('Undeploying %s webapp', name)
-        instance.undeploy_webapp(name)
+        instance.undeploy_webapp(
+            name,
+            wait=wait,
+            max_wait=max_wait,
+            timeout=timeout)
 
 
 class ACMEMetadataCLI(pki.cli.CLI):
